import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;



/**
 * @author Gregor Broll, Alex De Luca
 * 
 * This class prepares the connection with a bluetooth device (the gps-receiver) and displays its data
 */
public class BTConnection implements Runnable, CommandListener
{
	private BTGPSMidlet midlet;
	private String btURL;		
	private Display display;
	
	private Logger l;
	private Thread runner;
	private GPS gps;	
	
	private Command exitCommand = new Command("Exit", Command.SCREEN, 0);
	private Command pauseCommand = new Command("Pause", Command.SCREEN, 2);
	private Command resumeCommand = new Command("Resume", Command.SCREEN, 2);
	
	
	/**
	 * Constructor; sets up the interface and start
	 * 
	 * @param midlet
	 */
	public BTConnection(BTGPSMidlet midlet, String friendlyName, String btAddress)
	{
		this.midlet = midlet;
		btURL = "btspp://" + btAddress + ":1";
		display = Display.getDisplay(midlet);
		
		l = new Logger("Bluetooth Test", 512, 0);
		l.addCommand(exitCommand);
		l.addCommand(pauseCommand);
		l.addCommand(resumeCommand);
		l.setCommandListener(this);

		display.setCurrent(l);

		gps = new GPS(l, btURL);
		gps.start();

		start();
	}


	/**
	 * Shows and updates the data from the gps-receiver on the interface
	 */
	public void run()
	{
		while (Thread.currentThread() == runner)
		{
			try
			{
				Record record = gps.getRecord();
				l.setTitle("W: " + record.warning);
				l.setString(record.dateTimeOfFix + "\n" + record.lattitude + "," + 	record.longitude + ", " + record.quality + ", " + record.satelliteCount);
			}
			catch (EmptyBufferException e)
			{
				l.setString("No data received yet.");
			}

			try
			{
				Thread.sleep(3000);
			}
			catch (InterruptedException e)
			{
			}
		}
	}
	
	
	/**
	 * Command-handler
	 */
	public void commandAction(Command c, Displayable s)
	{
		if (c == exitCommand)
		{
			destroyApp(false);
			midlet.notifyDestroyed();
		}
		else if (c == pauseCommand)
		{
			stop();
			gps.stop();
		}
		else if (c == resumeCommand)
		{
			start();
			gps.start();
		}
	}

	
	/**
	 * 
	 */
	public void start()
	{
		if (runner == null)
		{
			runner = new Thread(this);
			runner.start();
		}
	}

	
	/**
	 * 
	 */
	public void stop()
	{
		runner = null;
	}
	
	
	/**
	 * 
	 */
	public void pauseApp()
	{
	}

	
	/**
	 * @param unconditional
	 */
	public void destroyApp(boolean unconditional)
	{
		stop();
		gps.stop();
	}
}
